@@ -1,82 +1,13 @@
-#![allow(dead_code)]
+//! Common structures shared between [super::manager::Manager] and [super::gui::GuiLoop].
+
+use super::entities;
+use super::gui;
 
 use ::slice_of_array::prelude::*;
-use enum_as_inner::EnumAsInner;
-
-pub struct GuiData {
-    pub components: linked_hash_map::LinkedHashMap<String, Box<dyn Component>>,
-    pub widgets: linked_hash_map::LinkedHashMap<String, Box<dyn Widget>>,
-}
-
-impl GuiData {
-    pub fn default() -> Self {
-        Self {
-            components: linked_hash_map::LinkedHashMap::new(),
-            widgets: linked_hash_map::LinkedHashMap::new(),
-        }
-    }
-}
-
-pub trait FromGuiLoopMessage: Send {
-    fn update(&self, components: &mut linked_hash_map::LinkedHashMap<String, Box<dyn Component>>);
-}
-
-pub struct UpdateEnumStringRepr {
-    pub label: String,
-    pub value: String,
-}
-
-impl FromGuiLoopMessage for UpdateEnumStringRepr {
-    fn update(&self, components: &mut linked_hash_map::LinkedHashMap<String, Box<dyn Component>>) {
-        components
-            .get_mut(&self.label)
-            .unwrap()
-            .downcast_mut::<EnumStringRepr>()
-            .unwrap()
-            .value = self.value.clone();
-    }
-}
-
-pub struct UpdateValue<T> {
-    pub label: String,
-    pub value: T,
-}
-
-impl FromGuiLoopMessage for UpdateValue<bool> {
-    fn update(&self, components: &mut linked_hash_map::LinkedHashMap<String, Box<dyn Component>>) {
-        components
-            .get_mut(&self.label)
-            .unwrap()
-            .downcast_mut::<Var<bool>>()
-            .unwrap()
-            .value = self.value;
-    }
-}
-
-pub struct UpdateRangedValue<T> {
-    pub label: String,
-    pub value: T,
-}
-
-pub trait Numbers: egui::emath::Numeric + downcast_rs::DowncastSync {}
-
-impl Numbers for i32 {}
-impl Numbers for i64 {}
-impl Numbers for f32 {}
-impl Numbers for f64 {}
-
-impl<T: Numbers> FromGuiLoopMessage for UpdateRangedValue<T> {
-    fn update(&self, components: &mut linked_hash_map::LinkedHashMap<String, Box<dyn Component>>) {
-        components
-            .get_mut(&self.label)
-            .unwrap()
-            .downcast_mut::<RangedVar<T>>()
-            .unwrap()
-            .value = self.value;
-    }
-}
 
+/// Component such as a button or a slider.
 pub trait Component: downcast_rs::DowncastSync {
+    /// How to display the component on the side panel.
     fn show(
         &mut self,
         label: &str,
@@ -93,8 +24,11 @@ impl core::fmt::Debug for dyn Component {
 
 downcast_rs::impl_downcast!(sync Component);
 
+/// String representation of enum.
 pub struct EnumStringRepr {
+    /// String representation of the current value.
     pub value: String,
+    /// All possible values.
     pub values: std::vec::Vec<String>,
 }
 
@@ -127,7 +61,11 @@ impl Component for EnumStringRepr {
     }
 }
 
+/// Variable bool (checkbox) or numeric (read-only text box).
+///
+/// Interfaced by [super::manager::UiVar].
 pub struct Var<T> {
+    /// Current value.
     pub value: T,
 }
 
@@ -149,13 +87,55 @@ impl Component for Var<bool> {
     }
 }
 
+/// A button.
+///
+/// Interfaced by [super::manager::UiButton].
+pub struct Button {
+    /// Is true is recently pressed.
+    pub pressed: bool,
+}
+
+impl Component for Button {
+    fn show(
+        &mut self,
+        label: &str,
+        ui: &mut egui::Ui,
+        sender: &mut std::sync::mpsc::Sender<Box<dyn FromGuiLoopMessage>>,
+    ) {
+        if ui.button(label).clicked() {
+            sender
+                .send(Box::new(UpdateButton {
+                    label: label.to_string(),
+                }))
+                .unwrap();
+        }
+    }
+}
+
+impl<T: Number> Component for Var<T> {
+    fn show(
+        &mut self,
+        label: &str,
+        ui: &mut egui::Ui,
+        _sender: &mut std::sync::mpsc::Sender<Box<dyn FromGuiLoopMessage>>,
+    ) {
+        ui.label(format!("{}: {}", label, self.value));
+    }
+}
+
+/// A range value, represented as slider.
+///
+/// Interfaced by [super::manager::UiRangedVar].
 pub struct RangedVar<T> {
+    /// Current value.
     pub value: T,
+    /// Min bound.
     pub min: T,
+    /// Max bound.
     pub max: T,
 }
 
-impl<T: Numbers> Component for RangedVar<T> {
+impl<T: Number> Component for RangedVar<T> {
     fn show(
         &mut self,
         label: &str,
@@ -163,7 +143,7 @@ impl<T: Numbers> Component for RangedVar<T> {
         sender: &mut std::sync::mpsc::Sender<Box<dyn FromGuiLoopMessage>>,
     ) {
         if ui
-            .add(egui::Slider::new(&mut self.value, self.min..=self.max))
+            .add(egui::Slider::new(&mut self.value, self.min..=self.max).text(label))
             .changed()
         {
             sender
@@ -176,89 +156,21 @@ impl<T: Numbers> Component for RangedVar<T> {
     }
 }
 
-pub trait ToGuiLoopMessage: Send {
-    fn update_gui(self: Box<Self>, data: &mut GuiData, ctx: &mut miniquad::Context);
-}
-
-pub struct AddEnumStringRepr {
-    pub label: String,
-    pub value: String,
-    pub values: std::vec::Vec<String>,
-}
-
-impl ToGuiLoopMessage for AddEnumStringRepr {
-    fn update_gui(self: Box<Self>, data: &mut GuiData, _ctx: &mut miniquad::Context) {
-        data.components.insert(
-            self.label,
-            Box::new(EnumStringRepr {
-                value: self.value,
-                values: self.values,
-            }),
-        );
-    }
-}
-
-pub struct AddVar<T> {
-    pub label: String,
-    pub value: T,
-}
-
-impl ToGuiLoopMessage for AddVar<bool> {
-    fn update_gui(self: Box<Self>, data: &mut GuiData, _ctx: &mut miniquad::Context) {
-        data.components
-            .insert(self.label, Box::new(Var::<bool> { value: self.value }));
-    }
-}
-
-pub struct AddRangedVar<T> {
-    pub label: String,
-    pub value: T,
-    pub min: T,
-    pub max: T,
-}
-
-impl<T: Numbers> ToGuiLoopMessage for AddRangedVar<T> {
-    fn update_gui(self: Box<Self>, data: &mut GuiData, _ctx: &mut miniquad::Context) {
-        data.components.insert(
-            self.label,
-            Box::new(RangedVar::<T> {
-                value: self.value,
-                min: self.min,
-                max: self.max,
-            }),
-        );
-    }
-}
-
-pub struct AddWidget3 {
-    pub label: String,
-}
-
-impl ToGuiLoopMessage for AddWidget3 {
-    fn update_gui(self: Box<Self>, data: &mut GuiData, ctx: &mut miniquad::Context) {
-        data.widgets.insert(self.label, Box::new(Widget3::new(ctx)));
-    }
-}
-
-pub struct DeleteComponent {
-    pub label: String,
-}
-
-impl ToGuiLoopMessage for DeleteComponent {
-    fn update_gui(self: Box<Self>, data: &mut GuiData, _ctx: &mut miniquad::Context) {
-        data.components.remove(&self.label);
-    }
-}
-
+/// Widget for main panel.
 pub trait Widget: downcast_rs::DowncastSync {
+    /// How to render.
     fn render(&mut self, ctx: &mut miniquad::Context);
 
+    /// How to display the rendered content.
     fn show(
         &mut self,
         ui: &mut egui::Ui,
         assigned_width: f32,
         assigned_height: f32,
     ) -> Option<egui::Response>;
+
+    /// The aspect ratio of self.
+    fn aspect_ratio(&self) -> f32;
 }
 
 downcast_rs::impl_downcast!(sync Widget);
@@ -301,14 +213,13 @@ mod offscreen_shader {
     }
 }
 
+/// [Widget] for 3d content such as meshes, line segments and point clouds.
 pub struct Widget3 {
-    pub entities: linked_hash_map::LinkedHashMap<String, NamedEntity3>,
+    entities: linked_hash_map::LinkedHashMap<String, entities::NamedEntity3>,
     offscreen_pipeline: miniquad::Pipeline,
     //offscreen_bind: miniquad::Bindings,
     offscreen_pass: miniquad::RenderPass,
     aspect_ratio: f32,
-    rx: f32,
-    ry: f32,
     texture_id: Option<egui::TextureId>,
 }
 
@@ -367,8 +278,6 @@ impl Widget3 {
             //offscreen_bind,
             offscreen_pass,
             aspect_ratio: 640.0 / 480.0,
-            rx: 0.,
-            ry: 0.,
             texture_id: None,
         }
     }
@@ -377,18 +286,12 @@ impl Widget3 {
 impl Widget for Widget3 {
     fn render(&mut self, ctx: &mut miniquad::Context) {
         let proj = nalgebra_glm::perspective_fov_rh(60.0f32.to_radians(), 640.0, 480.0, 0.01, 10.0);
-        let view = nalgebra_glm::look_at_rh(
-            &nalgebra_glm::vec3(0.0, 1.5, 3.0),
-            &nalgebra_glm::vec3(0.0, 0.0, 0.0),
-            &nalgebra_glm::vec3(0.0, 1.0, 0.0),
-        );
-        let view_proj = proj * view;
 
-        self.rx += 0.01;
-        self.ry += 0.03;
+        // TODO: implement mouse interaction / orbital control. For now the camera is placed at a
+        //       constant position looking at the scene.
         let camera_pose_scene = nalgebra::Isometry3::<f32>::from_parts(
-            nalgebra::Translation3::<f32>::new(0.0, 0.0, 0.0),
-            nalgebra::UnitQuaternion::<f32>::from_euler_angles(self.rx, self.ry, 0.),
+            nalgebra::Translation3::<f32>::new(0.0, 0.0, -4.0),
+            nalgebra::UnitQuaternion::<f32>::from_euler_angles(0.0, 0.0, 0.),
         );
 
         // the offscreen render pipeline, following this example:
@@ -426,7 +329,7 @@ impl Widget for Widget3 {
             ctx.apply_bindings(&offscreen_bind);
 
             let vs_params = offscreen_shader::Uniforms {
-                mvp: view_proj
+                mvp: proj
                     * camera_pose_scene.to_matrix()
                     * named_entity.scene_pose_entity.to_matrix(),
             };
@@ -463,166 +366,137 @@ impl Widget for Widget3 {
         );
         Some(r)
     }
-}
 
-#[repr(C)]
-struct Color {
-    r: f32,
-    g: f32,
-    b: f32,
-    alpha: f32,
-}
-
-struct PositionColorVertices {
-    vertices: std::vec::Vec<[f32; 7]>,
-}
-
-impl PositionColorVertices {
-    fn to_array(position: nalgebra::Vector3<f32>, color: Color) -> [f32; 7] {
-        [
-            position.x,
-            position.y,
-            position.z,
-            color.r,
-            color.g,
-            color.b,
-            color.alpha,
-        ]
+    fn aspect_ratio(&self) -> f32 {
+        self.aspect_ratio
     }
 }
 
-#[repr(C)]
-struct PositionUvVertices {
-    vertices: std::vec::Vec<[f32; 5]>,
+/// Integer or floating point number.
+
+pub trait Number: egui::emath::Numeric + downcast_rs::DowncastSync + std::fmt::Display {}
+
+impl Number for usize {}
+impl Number for i32 {}
+impl Number for i64 {}
+impl Number for f32 {}
+impl Number for f64 {}
+
+/// Message from  [super::manager::Manager] to [super::gui::GuiLoop], such as to add a component or
+/// widget.
+pub trait ToGuiLoopMessage: Send {
+    /// How that component or widget shall be displayed.
+    fn update_gui(self: Box<Self>, data: &mut gui::GuiData, ctx: &mut miniquad::Context);
 }
 
-impl PositionUvVertices {
-    fn to_array(position: nalgebra::Vector3<f32>, uv: nalgebra::Vector2<f32>) -> [f32; 5] {
-        [position.x, position.y, position.z, uv.x, uv.y]
-    }
-}
-
-pub struct Texture {}
-
-struct PositionUvVerticesAndTexture {
-    vertices: PositionUvVertices,
-    texture: Texture,
-}
-
-#[derive(EnumAsInner)]
-enum Vertices {
-    PositionColor(PositionColorVertices),
-    PositionUvAndTexture(PositionUvVerticesAndTexture),
-}
-
-pub struct Entity3 {
-    vertices: Vertices,
-    faces: Faces,
-}
-
-pub struct NamedEntity3 {
+/// Add an enum (as string representation) as combo box to side panel.
+pub struct AddEnumStringRepr {
+    /// The name of the enum.
     pub label: String,
-    pub entity: Entity3,
-    pub scene_pose_entity: nalgebra::Isometry3<f32>,
+    /// Initial value.
+    pub value: String,
+    /// List of possible values.
+    pub values: std::vec::Vec<String>,
 }
 
-impl Entity3 {
-    fn from_position_color_vertices_and_faces(
-        vertices: PositionColorVertices,
-        faces: Faces,
-    ) -> Self {
-        Self {
-            vertices: Vertices::PositionColor(vertices),
-            faces,
-        }
-    }
-
-    fn from_position_uv_vertices_texture_and_faces(
-        vertices: PositionUvVertices,
-        texture: Texture,
-        faces: Faces,
-    ) -> Self {
-        Self {
-            vertices: Vertices::PositionUvAndTexture(PositionUvVerticesAndTexture {
-                vertices,
-                texture,
+impl ToGuiLoopMessage for AddEnumStringRepr {
+    fn update_gui(self: Box<Self>, data: &mut gui::GuiData, _ctx: &mut miniquad::Context) {
+        data.components.insert(
+            self.label,
+            Box::new(EnumStringRepr {
+                value: self.value,
+                values: self.values,
             }),
-            faces,
-        }
+        );
     }
 }
 
-#[repr(C)]
-
-pub struct Faces {
-    indices: std::vec::Vec<[i16; 3]>,
+/// To add a button to side panel.
+pub struct AddButton {
+    /// The name of button.
+    pub label: String,
 }
 
-impl Faces {
-    fn new(indices: std::vec::Vec<[i16; 3]>) -> Self {
-        Self { indices }
+impl ToGuiLoopMessage for AddButton {
+    fn update_gui(self: Box<Self>, data: &mut gui::GuiData, _ctx: &mut miniquad::Context) {
+        data.components
+            .insert(self.label, Box::new(Button { pressed: false }));
     }
 }
 
-pub fn colored_cube(scale: f32) -> Entity3 {
-    #[rustfmt::skip]
-    let vertices = PositionColorVertices{vertices: vec![
-       [-scale, -scale, -scale,    1.0, 0.5, 0.5, 1.0],
-       [ scale, -scale, -scale,    1.0, 0.5, 0.5, 1.0],
-       [ scale,  scale, -scale,    1.0, 0.5, 0.5, 1.0],
-       [-scale,  scale, -scale,    1.0, 0.5, 0.5, 1.0],
-
-       [-scale, -scale,  scale,    0.5, 1.0, 0.5, 1.0],
-       [ scale, -scale,  scale,    0.5, 1.0, 0.5, 1.0],
-       [ scale,  scale,  scale,    0.5, 1.0, 0.5, 1.0],
-       [-scale,  scale,  scale,    0.5, 1.0, 0.5, 1.0],
-  
-       [-scale, -scale, -scale,    0.5, 0.5, 1.0, 1.0],
-       [-scale,  scale, -scale,    0.5, 0.5, 1.0, 1.0],
-       [-scale,  scale,  scale,    0.5, 0.5, 1.0, 1.0],
-       [-scale, -scale,  scale,    0.5, 0.5, 1.0, 1.0],
-  
-       [ scale, -scale, -scale,    1.0, 0.5, 0.0, 1.0],
-       [ scale,  scale, -scale,    1.0, 0.5, 0.0, 1.0],
-       [ scale,  scale,  scale,    1.0, 0.5, 0.0, 1.0],
-       [ scale, -scale,  scale,    1.0, 0.5, 0.0, 1.0],
-       
-       [-scale, -scale, -scale,    0.0, 0.5, 1.0, 1.0],
-       [-scale, -scale,  scale,    0.0, 0.5, 1.0, 1.0],
-       [ scale, -scale,  scale,    0.0, 0.5, 1.0, 1.0],
-       [ scale, -scale, -scale,    0.0, 0.5, 1.0, 1.0],
-         
-       [-scale,  scale, -scale,    1.0, 0.0, 0.5, 1.0],
-       [-scale,  scale,  scale,    1.0, 0.0, 0.5, 1.0],
-       [ scale,  scale,  scale,    1.0, 0.0, 0.5, 1.0],
-       [ scale,  scale, -scale,    1.0, 0.0, 0.5, 1.0],
-    ]};
-
-    let faces = Faces::new(vec![
-        [0, 1, 2],
-        [0, 2, 3],
-        [6, 5, 4],
-        [7, 6, 4],
-        [8, 9, 10],
-        [8, 10, 11],
-        [14, 13, 12],
-        [15, 14, 12],
-        [16, 17, 18],
-        [16, 18, 19],
-        [22, 21, 20],
-        [23, 22, 20],
-    ]);
-
-    Entity3::from_position_color_vertices_and_faces(vertices, faces)
+/// Add bool (as checkbox) or numeric value (as read-only text box) to side panel.
+///
+/// Also see [Var].
+pub struct AddVar<T> {
+    /// The name of variable.
+    pub label: String,
+    /// The initial value.
+    pub value: T,
 }
 
-pub struct PlaceEntity {
+impl ToGuiLoopMessage for AddVar<bool> {
+    fn update_gui(self: Box<Self>, data: &mut gui::GuiData, _ctx: &mut miniquad::Context) {
+        data.components
+            .insert(self.label, Box::new(Var::<bool> { value: self.value }));
+    }
+}
+
+impl<T: Number> ToGuiLoopMessage for AddVar<T> {
+    fn update_gui(self: Box<Self>, data: &mut gui::GuiData, _ctx: &mut miniquad::Context) {
+        data.components
+            .insert(self.label, Box::new(Var::<T> { value: self.value }));
+    }
+}
+
+/// Add a numeric value as a slider to side panel.
+///
+/// Also see [RangedVar].
+pub struct AddRangedVar<T> {
+    /// Name of variable.
+    pub label: String,
+    /// Initial value.
+    pub value: T,
+    /// Min bounds
+    pub min: T,
+    /// Max bounds
+    pub max: T,
+}
+
+impl<T: Number> ToGuiLoopMessage for AddRangedVar<T> {
+    fn update_gui(self: Box<Self>, data: &mut gui::GuiData, _ctx: &mut miniquad::Context) {
+        data.components.insert(
+            self.label,
+            Box::new(RangedVar::<T> {
+                value: self.value,
+                min: self.min,
+                max: self.max,
+            }),
+        );
+    }
+}
+
+/// Adds [Widget3] to main panel.
+pub struct AddWidget3 {
+    /// Name of widget
+    pub label: String,
+}
+
+impl ToGuiLoopMessage for AddWidget3 {
+    fn update_gui(self: Box<Self>, data: &mut gui::GuiData, ctx: &mut miniquad::Context) {
+        data.widgets.insert(self.label, Box::new(Widget3::new(ctx)));
+    }
+}
+
+/// Place [super::entities::Entity3] in corresponding [Widget3].
+pub struct PlaceEntity3 {
+    /// Name of widget.
     pub widget_label: String,
-    pub named_entity: NamedEntity3,
+    /// The 3d entity
+    pub named_entity: entities::NamedEntity3,
 }
 
-impl ToGuiLoopMessage for PlaceEntity {
-    fn update_gui(self: Box<Self>, data: &mut GuiData, _ctx: &mut miniquad::Context) {
+impl ToGuiLoopMessage for PlaceEntity3 {
+    fn update_gui(self: Box<Self>, data: &mut gui::GuiData, _ctx: &mut miniquad::Context) {
         data.widgets
             .get_mut(&self.widget_label)
             .unwrap()
@@ -632,3 +506,133 @@ impl ToGuiLoopMessage for PlaceEntity {
             .insert(self.named_entity.label.clone(), self.named_entity);
     }
 }
+
+/// Updates pose of [super::entities::Entity3] in corresponding [Widget3].
+///
+/// It is no-op, if an entity with that name `entity_label` does not exist.
+pub struct UpdateScenePoseEntity3 {
+    /// Name of widget.
+    pub widget_label: String,
+    /// Name of entity.
+    pub entity_label: String,
+    /// Pose of the entity in the scene.
+    pub scene_pose_entity: nalgebra::Isometry3<f32>,
+}
+
+impl ToGuiLoopMessage for UpdateScenePoseEntity3 {
+    fn update_gui(self: Box<Self>, data: &mut gui::GuiData, _ctx: &mut miniquad::Context) {
+        let maybe_entity = data
+            .widgets
+            .get_mut(&self.widget_label)
+            .unwrap()
+            .downcast_mut::<Widget3>()
+            .unwrap()
+            .entities
+            .get_mut(&self.entity_label);
+        if maybe_entity.is_none() {
+            // No-op.
+            return;
+        }
+        maybe_entity.unwrap().scene_pose_entity = self.scene_pose_entity;
+    }
+}
+
+/// Delete that component from side panel.
+pub struct DeleteComponent {
+    /// Name/identifier of component
+    pub label: String,
+}
+
+impl ToGuiLoopMessage for DeleteComponent {
+    fn update_gui(self: Box<Self>, data: &mut gui::GuiData, _ctx: &mut miniquad::Context) {
+        data.components.remove(&self.label);
+    }
+}
+
+/// Message from [super::gui::GuiLoop] to [super::manager::Manager].
+pub trait FromGuiLoopMessage: Send {
+    /// How to update the state given user interactions (button presses etc.).
+    fn update(&self, components: &mut linked_hash_map::LinkedHashMap<String, Box<dyn Component>>);
+}
+
+/// [super::manager::UiEnum]  (i.e. slider) update.
+///
+/// See also [EnumStringRepr].
+pub struct UpdateEnumStringRepr {
+    /// The name.
+    pub label: String,
+    /// The new value.
+    pub value: String,
+}
+
+impl FromGuiLoopMessage for UpdateEnumStringRepr {
+    fn update(&self, components: &mut linked_hash_map::LinkedHashMap<String, Box<dyn Component>>) {
+        components
+            .get_mut(&self.label)
+            .unwrap()
+            .downcast_mut::<EnumStringRepr>()
+            .unwrap()
+            .value = self.value.clone();
+    }
+}
+
+/// [Var] update.
+///
+/// See also [super::manager::UiVar].
+pub struct UpdateValue<T> {
+    /// The name.
+    pub label: String,
+    /// The new value.
+    pub value: T,
+}
+
+impl FromGuiLoopMessage for UpdateValue<bool> {
+    fn update(&self, components: &mut linked_hash_map::LinkedHashMap<String, Box<dyn Component>>) {
+        components
+            .get_mut(&self.label)
+            .unwrap()
+            .downcast_mut::<Var<bool>>()
+            .unwrap()
+            .value = self.value;
+    }
+}
+
+/// [RangedVar] (slider) update.
+///
+/// See also [super::manager::UiRangedVar].
+pub struct UpdateRangedValue<T> {
+    /// The name.
+    pub label: String,
+    /// The new value.
+    pub value: T,
+}
+
+impl<T: Number> FromGuiLoopMessage for UpdateRangedValue<T> {
+    fn update(&self, components: &mut linked_hash_map::LinkedHashMap<String, Box<dyn Component>>) {
+        components
+            .get_mut(&self.label)
+            .unwrap()
+            .downcast_mut::<RangedVar<T>>()
+            .unwrap()
+            .value = self.value;
+    }
+}
+
+/// [Button] press event.
+///
+/// See also [super::manager::UiButton].
+pub struct UpdateButton {
+    /// The name.
+    pub label: String,
+}
+
+impl FromGuiLoopMessage for UpdateButton {
+    fn update(&self, components: &mut linked_hash_map::LinkedHashMap<String, Box<dyn Component>>) {
+        components
+            .get_mut(&self.label)
+            .unwrap()
+            .downcast_mut::<Button>()
+            .unwrap()
+            .pressed = true;
+    }
+}
